@@ -7,73 +7,13 @@ import shutil
 import subprocess
 import tempfile
 
-from concurrent.futures import ThreadPoolExecutor, as_completed
-from datetime import datetime
-from functools import partial
+from pathlib import Path
 from urllib.parse import urlparse
 
-from twitchdl import twitch
-from twitchdl.download import download_file
+from twitchdl import twitch, utils
+from twitchdl.download import download_file, download_files
 from twitchdl.exceptions import ConsoleError
-from twitchdl.output import print_out
-from twitchdl.utils import slugify
-
-
-def read_int(msg, min, max, default):
-    msg = msg + " [default {}]: ".format(default)
-
-    while True:
-        try:
-            val = input(msg)
-            if not val:
-                return default
-            if min <= int(val) <= max:
-                return int(val)
-        except ValueError:
-            pass
-
-
-def format_size(bytes_):
-    if bytes_ < 1024:
-        return str(bytes_)
-
-    kilo = bytes_ / 1024
-    if kilo < 1024:
-        return "{:.1f}K".format(kilo)
-
-    mega = kilo / 1024
-    if mega < 1024:
-        return "{:.1f}M".format(mega)
-
-    return "{:.1f}G".format(mega / 1024)
-
-
-def format_duration(total_seconds):
-    total_seconds = int(total_seconds)
-    hours = total_seconds // 3600
-    remainder = total_seconds % 3600
-    minutes = remainder // 60
-    seconds = total_seconds % 60
-
-    if hours:
-        return "{} h {} min".format(hours, minutes)
-
-    if minutes:
-        return "{} min {} sec".format(minutes, seconds)
-
-    return "{} sec".format(seconds)
-
-
-def _print_video(video):
-    published_at = video['published_at'].replace('T', ' @ ').replace('Z', '')
-    length = format_duration(video['length'])
-    name = video['channel']['display_name']
-
-    print_out("\n<bold>{}</bold>".format(video['_id'][1:]))
-    print_out("<green>{}</green>".format(video["title"]))
-    print_out("<cyan>{}</cyan> playing <cyan>{}</cyan>".format(name, video['game']))
-    print_out("Published <cyan>{}</cyan>  Length: <cyan>{}</cyan> ".format(published_at, length))
-    print_out("<i>{}</i>".format(video["url"]))
+from twitchdl.output import print_out, print_video
 
 
 def videos(channel_name, limit, offset, sort, **kwargs):
@@ -95,7 +35,7 @@ def videos(channel_name, limit, offset, sort, **kwargs):
     print_out("<yellow>Showing videos {}-{} of {}</yellow>".format(first, last, total))
 
     for video in videos['videos']:
-        _print_video(video)
+        print_video(video)
 
 
 def _select_quality(playlists):
@@ -105,49 +45,16 @@ def _select_quality(playlists):
         resolution = "x".join(str(r) for r in p.stream_info.resolution)
         print_out("{}) {} [{}]".format(n + 1, name, resolution))
 
-    no = read_int("Choose quality", min=1, max=len(playlists) + 1, default=1)
+    no = utils.read_int("Choose quality", min=1, max=len(playlists) + 1, default=1)
 
     return playlists[no - 1]
 
 
-def _print_progress(futures):
-    counter = 1
-    total = len(futures)
-    total_size = 0
-    start_time = datetime.now()
-
-    for future in as_completed(futures):
-        size = future.result()
-        percentage = 100 * counter // total
-        total_size += size
-        duration = (datetime.now() - start_time).seconds
-        speed = total_size // duration if duration else 0
-        remaining = (total - counter) * duration / counter
-
-        msg = "Downloaded VOD {}/{} ({}%) total <cyan>{}B</cyan> at <cyan>{}B/s</cyan> remaining <cyan>{}</cyan>".format(
-            counter, total, percentage, format_size(total_size), format_size(speed), format_duration(remaining))
-
-        print_out("\r" + msg.ljust(80), end='')
-        counter += 1
-
-
-def _download_files(base_url, directory, filenames, max_workers):
-    urls = [base_url + f for f in filenames]
-    paths = ["/".join([directory, f]) for f in filenames]
-    partials = (partial(download_file, url, path) for url, path in zip(urls, paths))
-
-    with ThreadPoolExecutor(max_workers=max_workers) as executor:
-        futures = [executor.submit(fn) for fn in partials]
-        _print_progress(futures)
-
-    return paths
-
-
-def _join_vods(directory, paths, target):
+def _join_vods(directory, file_paths, target):
     input_path = "{}/files.txt".format(directory)
 
     with open(input_path, 'w') as f:
-        for path in paths:
+        for path in file_paths:
             f.write('file {}\n'.format(os.path.basename(path)))
 
     result = subprocess.run([
@@ -171,24 +78,12 @@ def _video_target_filename(video, format):
         date,
         video['_id'][1:],
         video['channel']['name'],
-        slugify(video['title']),
+        utils.slugify(video['title']),
     ])
 
     return name + "." + format
 
 
-def _parse_video_id(video_id):
-    """This can be either a integer ID or an URL to the video on twitch."""
-    if re.search(r"^\d+$", video_id):
-        return int(video_id)
-
-    match = re.search(r"^https://www.twitch.tv/videos/(\d+)(\?.+)?$", video_id)
-    if match:
-        return int(match.group(1))
-
-    raise ConsoleError("Invalid video ID given, expected integer ID or Twitch URL")
-
-
 def _get_files(playlist, start, end):
     """Extract files for download from playlist."""
     vod_start = 0
@@ -214,9 +109,69 @@ def _crete_temp_dir(base_uri):
     return directory
 
 
-def download(video_id, max_workers, format='mkv', start=None, end=None, keep=False, **kwargs):
-    video_id = _parse_video_id(video_id)
+VIDEO_PATTERNS = [
+    r"^(?P<id>\d+)?$",
+    r"^https://www.twitch.tv/videos/(?P<id>\d+)(\?.+)?$",
+]
 
+CLIP_PATTERNS = [
+    r"^(?P<slug>[A-Za-z]+)$",
+    r"^https://www.twitch.tv/\w+/clip/(?P<slug>[A-Za-z]+)(\?.+)?$",
+    r"^https://clips.twitch.tv/(?P<slug>[A-Za-z]+)(\?.+)?$",
+]
+
+
+def download(video, **kwargs):
+    for pattern in CLIP_PATTERNS:
+        match = re.match(pattern, video)
+        if match:
+            clip_slug = match.group('slug')
+            return _download_clip(clip_slug, **kwargs)
+
+    for pattern in VIDEO_PATTERNS:
+        match = re.match(pattern, video)
+        if match:
+            video_id = match.group('id')
+            return _download_video(video_id, **kwargs)
+
+    raise ConsoleError("Invalid video: {}".format(video_id))
+
+
+def _download_clip(slug, **kwargs):
+    print_out("Looking up clip...")
+    clip = twitch.get_clip(slug)
+
+    print_out("Found: <green>{}</green> by <yellow>{}</yellow>, playing <blue>{}</blue> ({})".format(
+        clip["title"],
+        clip["broadcaster"]["displayName"],
+        clip["game"]["name"],
+        utils.format_duration(clip["durationSeconds"])
+    ))
+
+    print_out("\nAvailable qualities:")
+    qualities = clip["videoQualities"]
+    for n, q in enumerate(qualities):
+        print_out("{}) {} [{} fps]".format(n + 1, q["quality"], q["frameRate"]))
+
+    no = utils.read_int("Choose quality", min=1, max=len(qualities), default=1)
+    selected_quality = qualities[no - 1]
+    url = selected_quality["sourceURL"]
+
+    url_path = urlparse(url).path
+    extension = Path(url_path).suffix
+    filename = "{}_{}{}".format(
+        clip["broadcaster"]["login"],
+        utils.slugify(clip["title"]),
+        extension
+    )
+
+    print("Downloading clip...")
+    download_file(url, filename)
+
+    print("Downloaded: {}".format(filename))
+
+
+def _download_video(video_id, max_workers, format='mkv', start=None, end=None, keep=False, **kwargs):
     if start and end and end <= start:
         raise ConsoleError("End time must be greater than start time")
 
@@ -251,11 +206,11 @@ def download(video_id, max_workers, format='mkv', start=None, end=None, keep=Fal
 
     print_out("\nDownloading {} VODs using {} workers to {}".format(
         len(filenames), max_workers, target_dir))
-    _download_files(base_uri, target_dir, filenames, max_workers)
+    file_paths = download_files(base_uri, target_dir, filenames, max_workers)
 
     print_out("\n\nJoining files...")
     target = _video_target_filename(video, format)
-    _join_vods(target_dir, filenames, target)
+    _join_vods(target_dir, file_paths, target)
 
     if keep:
         print_out("\nTemporary files not deleted: {}".format(target_dir))
