@@ -21,6 +21,19 @@ def authenticated_get(url, params={}, headers={}):
     return response
 
 
+def authenticated_post(url, data=None, json=None, headers={}):
+    headers['Client-ID'] = CLIENT_ID
+
+    response = requests.post(url, data=data, json=json, headers=headers)
+    if response.status_code == 400:
+        data = response.json()
+        raise ConsoleError(data["message"])
+
+    response.raise_for_status()
+
+    return response
+
+
 def kraken_get(url, params={}, headers={}):
     """
     Add accept header required by kraken API v5.
@@ -46,11 +59,40 @@ def get_video(video_id):
     """
     https://dev.twitch.tv/docs/v5/reference/videos#get-video
     """
-    url = "https://api.twitch.tv/kraken/videos/%d" % video_id
+    url = "https://api.twitch.tv/kraken/videos/{}".format(video_id)
 
     return kraken_get(url).json()
 
 
+def get_clip(slug):
+    url = "https://gql.twitch.tv/gql"
+
+    query = """
+    {{
+        clip(slug: "{}") {{
+            title
+            durationSeconds
+            game {{
+                name
+            }}
+            broadcaster {{
+                login
+                displayName
+            }}
+            videoQualities {{
+                frameRate
+                quality
+                sourceURL
+            }}
+        }}
+    }}
+    """
+
+    payload = {"query": query.format(slug)}
+    data = authenticated_post(url, json=payload).json()
+    return data["data"]["clip"]
+
+
 def get_channel_videos(channel_id, limit, offset, sort):
     """
     https://dev.twitch.tv/docs/v5/reference/channels#get-channel-videos
@@ -66,7 +108,7 @@ def get_channel_videos(channel_id, limit, offset, sort):
 
 
 def get_access_token(video_id):
-    url = "https://api.twitch.tv/api/vods/%d/access_token" % video_id
+    url = "https://api.twitch.tv/api/vods/{}/access_token".format(video_id)
 
     return authenticated_get(url).json()
 
